@@ -2,9 +2,9 @@
 ##  amelia diagnostic functins
 ##
 ##  05/05/06 mb - added amelia.arg compatibility
-##  07/05/06 jh - compare: changed how variable names are found, changed titles/labels, set x-axis values in matplot, colours for no imputations 
+##  07/05/06 jh - compare: changed how variable names are found, changed titles/labels, set x-axis values in matplot, colours for no imputations
 ##                overimpute: added new m-name in output,
-##  09/05/06 mb - overimpute: added frontend check for overimpute. 
+##  09/05/06 mb - overimpute: added frontend check for overimpute.
 ##  15/05/06 jh - overimpute: stacking of original data, and various graphics adjustments
 ##  01/06/06 mb - added "gethull" and "disperse" for overdispersion diagnostic
 ##  19/07/06 mb - moved handling of arglists to prep.
@@ -21,14 +21,16 @@
 ##                uses amelia class
 ##                overimpute: uses amelia class, added lwd, col, main, lab, etc for user
 ##                disperse: now uses amelia class
-
+##  02/21/12 jh - added mi.meld to combine multiply imputed quantities of interest and se's.
+##  10/30/12 jh - tscsPlot: expanded to allow to cycle through sets of cross sectional units efficiently.
 
 compare.density <- function(output,var,col=c("red","black"),scaled=FALSE,lwd=1,main,xlab,ylab,legend=TRUE,frontend=FALSE,...) {
 
   if (!("amelia" %in% class(output)))
     stop("The 'output' is not Amelia output.")
 
-  data <- getOriginalData(output)
+  ##data <- getOriginalData(output)
+  data <- remove.imputations(output)
 
   ## Checks on if the variable makes sense to plot.
   if (class(var)=="character")
@@ -40,7 +42,7 @@ compare.density <- function(output,var,col=c("red","black"),scaled=FALSE,lwd=1,m
     stop("The 'var' option points to a non-existant column.")
   if (var %in% output$arguments$idvar)
     stop("the variable selected was marked as an idvar")
-  
+
   ## We need to clean the data to make sure that
   ## we're not going to run into NAs
   mcount <- sum(!is.na(output$imputations))
@@ -48,7 +50,7 @@ compare.density <- function(output,var,col=c("red","black"),scaled=FALSE,lwd=1,m
 
   ## create an empty vector to sum across
   varimp <- matrix(NA, nrow(data), mcount)
-  
+
   for (i in 1:mcount) {
     varimp[,i] <- output$imputations[[imputed[i]]][,var]
   }
@@ -59,26 +61,26 @@ compare.density <- function(output,var,col=c("red","black"),scaled=FALSE,lwd=1,m
     leg.text <- "Mean Imputations"
     varimp <- rowMeans(varimp)
   }
-  
+
   if (frontend)
     x11()
-      
-  
+
+
   vars <- data[,var]
-  if (scaled) 
+  if (scaled)
     ratio <- sum(is.na(vars))/sum(!is.na(vars))
   else
     ratio <- 1
   varnames<-dimnames(data)[[2]]            # This will work for both data.frames AND matricies.
   vname<-varnames[var]                     # This will work for both data.frames AND matricies.
 
-  
+
   if (sum(is.na(vars)) > 0) {
     oiDetect <- (sum(output$missMatrix[,var]) + sum(!is.na(vars))) > length(vars)
     if (missing(main)) {
       if (oiDetect) {
         main <-  paste("Observed and Overimputed values of",vname)
-      } else { 
+      } else {
         main <- paste("Observed and Imputed values of",vname)
       }
     }
@@ -89,7 +91,7 @@ compare.density <- function(output,var,col=c("red","black"),scaled=FALSE,lwd=1,m
     if (missing(ylab)) {
       ylab <- "Relative Density"
     }
-    
+
     xmiss <- density(varimp[output$missMatrix[,var]],na.rm=TRUE)
     xobs  <- density(vars[!is.na(vars)],na.rm=TRUE)
     compplot <- matplot(x=cbind(xmiss$x,xobs$x),y=cbind(ratio*xmiss$y,xobs$y), xlab=xlab, ylab=ylab,type="l",lwd=lwd, lty=1,main=main,col=col,...)
@@ -107,9 +109,9 @@ compare.density <- function(output,var,col=c("red","black"),scaled=FALSE,lwd=1,m
     if (missing(ylab)) {
       ylab <- "Relative Density"
     }
-    
+
     compplot <- plot(density(varimp,na.rm=TRUE), col = "blue",
-      main = main,...)
+                     main = main,...)
     col.none=c("gray","blue")
 
     if (legend) {
@@ -117,7 +119,7 @@ compare.density <- function(output,var,col=c("red","black"),scaled=FALSE,lwd=1,m
              col=col.none,lty=c(1,1),bg='gray90')
     }
   }
-  
+
   invisible()
 }
 
@@ -127,6 +129,7 @@ compare.density <- function(output,var,col=c("red","black"),scaled=FALSE,lwd=1,m
 ##
 ##  INPUTS:output         - output from amelia run (class "amelia")
 ##         var            - column number or variable name to overimpute
+##         subset         - subset to overimpute and plot
 ##         legend         - should we add a lengend to the plot? (TRUE/FALSE)
 ##         xlab,ylab,main - graphical parameters
 ##         frontend       - logical for printing to tcl/tk window
@@ -136,16 +139,35 @@ compare.density <- function(output,var,col=c("red","black"),scaled=FALSE,lwd=1,m
 ##
 
 
-overimpute <- function(output,var,legend=TRUE,xlab,ylab,main,frontend=FALSE,...) {
+overimpute <- function(output, var, subset, legend = TRUE, xlab, ylab,
+                       main, frontend = FALSE,...) {
 
   if (!("amelia" %in% class(output)))
     stop("The 'output' is not Amelia output.")
 
-  
+
   data <- getOriginalData(output)
+
+  ## via the subset.data.frame function
+  if (missing(subset)) {
+    r <- TRUE
+  } else {
+    e <- substitute(subset)
+    r <- eval(e, data, parent.frame())
+    if (!is.logical(r)) {
+      stop("'subset' must evaluate to logical")
+    }
+    r <- r & !is.na(r)
+    if (sum(r) == 0) {
+      stop("no observations in the subset")
+    }
+  }
+  data <- data[r,]
+
   origAMr1 <- is.na(data)
 
-  # Allow character names as arguments for "var" with data.frames
+
+  ## Allow character names as arguments for "var" with data.frames
 
   if(is.character(var)){
     if(!is.data.frame(data)){
@@ -155,7 +177,7 @@ overimpute <- function(output,var,legend=TRUE,xlab,ylab,main,frontend=FALSE,...)
       if(is.na(varpos)){
         stop("The name provided for var argument does not exist in the dataset provided.")
       } else {
-      var<-varpos
+        var<-varpos
       }
     }
   }
@@ -167,22 +189,23 @@ overimpute <- function(output,var,legend=TRUE,xlab,ylab,main,frontend=FALSE,...)
 
   stacked.var<-match(var,prepped$subset.index[prepped$p.order])
   subset.var<-match(var,prepped$subset.index)
-  if (!is.null(prepped$blanks)) 
+  if (!is.null(prepped$blanks))
     fully.missing <- origAMr1[-prepped$blanks, var][prepped$n.order]
   else
     fully.missing <- origAMr1[, var][prepped$n.order]
-  
+
   if (is.na(stacked.var)) {
     if (frontend)
       tkmessageBox(message="The variable you selected doesn't exist in the Amelia output becuase it wasn't imputed.",icon="error",type="ok")
     stop("var doesn't exist in the amelia output.  It either didn't get imputed or is out of the range of columns.")
-  } 
-  
-  means<-c()
-  lowers<-c()
-  uppers<-c()
-  color<-c()
-  AMr1<-is.na(prepped$x)
+  }
+
+  means <- c()
+  lowers <- c()
+  uppers <- c()
+  pcnts <- c()
+  color <- c()
+  AMr1 <- is.na(prepped$x)
   ## if (sum(!AMr1[,stacked.var]) == 0){
   ##   if (frontend) {
   ##     tkmessageBox(parent = getAmelia("gui"),
@@ -194,59 +217,57 @@ overimpute <- function(output,var,legend=TRUE,xlab,ylab,main,frontend=FALSE,...)
   AMp<-ncol(prepped$x)
   for (i in 1:nrow(prepped$x)) {
     if (fully.missing[i]) {
-        next()
+      next()
     }
 
     x<-prepped$x[i,,drop=FALSE]
     x[1,stacked.var]<-NA
     o<-!is.na(x)
     miss<-!o
-    x[is.na(x)]<-0  
-    
-    #o<-!AMr1[i,]
-    #o[stacked.var]<-FALSE
-    
+    x[is.na(x)]<-0
+
+                                        #o<-!AMr1[i,]
+                                        #o[stacked.var]<-FALSE
+
     pcntmiss<-(sum(miss))/(length(miss)-sum(prepped$index==0))   # Does not include time polynomials (index==0) in the denominator
-                                                                 # These are always fully observed by construction, but auxiliary.
-                                                                 # Leaves constructed lags and leads, and nominal variables in count, however.
+    ## These are always fully observed by construction, but auxiliary.
+    ## Leaves constructed lags and
+    ## leads, and nominal variables
+    ## in count, however.
+
     conf<-c()
     for (k in 1:output$m) {
 
-      ## The theta matrix is now stored in a array with
+      ## The theta matrix is now stored in an array with
       ## dimensions c(vars+1,vars+1,m), so this grabs
       ## the kth theta matrix.
-      
+
       thetareal<-output$theta[,,k]
       theta <- amsweep(thetareal, c(FALSE,o))
       Ci<-matrix(0,AMp,AMp)
       hold<-chol(theta[c(FALSE,miss),c(FALSE,miss)])
-      Ci[miss,miss]<-hold      
+      Ci[miss,miss]<-hold
       imputations<-AMr1[i, , drop=FALSE] * ((x %*% theta[2:(AMp+1),2:(AMp+1) , drop=FALSE])
-          + (matrix(1,1,1) %*% theta[1,2:(AMp+1) , drop=FALSE]) )
-     ## for (j in 1:20) {             ## COULD REMOVE THIS LOOP 
-     ##                                 ##(mb: I removed it for speed)
-     ##   junk<-matrix(rnorm(AMp), 1, AMp) %*% Ci
-     ##   xc<-x + imputations + junk
-     ##   conf<-c(conf,xc[,stacked.var])      
-     ## }
+                               + (matrix(1,1,1) %*% theta[1,2:(AMp+1) , drop=FALSE]) )
+
       junk <- matrix(rnorm(20*AMp), 20, AMp) %*% Ci
       xc <- matrix(x,20,AMp,byrow=TRUE) +
-            matrix(imputations, 20, AMp, byrow=TRUE) +
-            junk
+        matrix(imputations, 20, AMp, byrow=TRUE) +
+          junk
       conf <- c(conf, xc[,stacked.var])
     }
-    
+
     scaled.conf <- (conf * prepped$scaled.sd[subset.var])  + prepped$scaled.mu[subset.var]
-    varlog<-match(var,prepped$logs)      
-    
+    varlog<-match(var,prepped$logs)
+
     if (!is.na(varlog))
       scaled.conf <- untransform(as.matrix(scaled.conf),logs=1,xmin=prepped$xmin[varlog],sqrts=NULL,lgstc=NULL)
     if (!is.na(match(var,prepped$sqrts)))
       scaled.conf <- untransform(as.matrix(scaled.conf),logs=NULL,xmin=NULL,sqrts=1,lgstc=NULL)
     if (!is.na(match(var,prepped$lgstc)))
       scaled.conf <- untransform(as.matrix(scaled.conf),logs=NULL,xmin=NULL,sqrts=NULL,lgstc=1)
-    
-    #colors are based on rainbow roygbiv l->r is higher missingness  \
+
+    ##colors are based on rainbow roygbiv l->r is higher missingness  \
     blue <- rgb(0,0,1, alpha = 0.75)
     green <- rgb(0,.75,0, alpha = 0.75)
     orange <- rgb(1, 0.65,0, alpha = 0.75)
@@ -267,18 +288,19 @@ overimpute <- function(output,var,legend=TRUE,xlab,ylab,main,frontend=FALSE,...)
     means<-c(means,mean(scaled.conf))
     lowers<-c(lowers,sort(scaled.conf)[round(output$m*20*0.05)])
     uppers<-c(uppers,sort(scaled.conf)[round(output$m*20*0.95)])
+    pcnts <- c(pcnts, pcntmiss)
 
   }
 
-  #AMr1<-is.na(prepped$x[,stacked.var])
-  #partial.n.order<-prepped$n.order[!origAMr1]
+                                        #AMr1<-is.na(prepped$x[,stacked.var])
+                                        #partial.n.order<-prepped$n.order[!origAMr1]
 
   if (is.null(prepped$blanks)) {
-    xplot<-data[prepped$n.order,var][!fully.missing]   
+    xplot<-data[prepped$n.order,var][!fully.missing]
   } else {
-    xplot<-data[-prepped$blanks,][prepped$n.order,var][!fully.missing]   
+    xplot<-data[-prepped$blanks,][prepped$n.order,var][!fully.missing]
   }
-  
+
 
   addedroom<-(max(uppers)-min(lowers))*0.1
 
@@ -296,7 +318,7 @@ overimpute <- function(output,var,legend=TRUE,xlab,ylab,main,frontend=FALSE,...)
     if (.Platform$OS.type == "windows") {
       windows()
     } else {
-      x11()      
+      x11()
     }
   }
   ci.order<-order(uppers-lowers,decreasing=TRUE)     # Allows smallest CI's to be printed last, and thus not buried in the plot.
@@ -307,11 +329,18 @@ overimpute <- function(output,var,legend=TRUE,xlab,ylab,main,frontend=FALSE,...)
     legend("bottomright",legend=c(" 0-.2",".2-.4",".4-.6",".6-.8",".8-1"),
            col=spectrum,lty=c(1,1),horiz=TRUE,bty="n")
   }
-  
+
   abline(0,1)
-  invisible(overplot)
+
+  out <- cbind(row = prepped$n.order[!fully.missing],
+               orig = xplot,
+               mean.overimputed = means,
+               lower.overimputed = lowers,
+               upper.overimputed = uppers,
+               prcntmiss = pcnts)
+  invisible(out)
 }
-      
+
 gethull <- function(st,tol,rots) {
   stvec<-st
   for (i in 1:length(st)) {
@@ -322,9 +351,9 @@ gethull <- function(st,tol,rots) {
   }
   reduced.hull<-t(rots)%*%stvec
   return(reduced.hull)
-}  
+}
+
 
-    
 disperse <- function(output, m = 5, dims = 1, p2s = 0, frontend=FALSE,...) {
 
   if (!("amelia" %in% class(output)))
@@ -333,13 +362,13 @@ disperse <- function(output, m = 5, dims = 1, p2s = 0, frontend=FALSE,...) {
   ## The original data is the imputed data with the
   ## imputations marked to NA. These two lines do that
   data <- getOriginalData(output)
-  
+
   if (frontend) {
     require(tcltk)
     putAmelia("output.log", c(getAmelia("output.log"), "==== Overdispersion Output ====\n"))
   }
 
-  # prep the data and arguments
+                                        # prep the data and arguments
   prepped<-amelia.prep(x=data, arglist=output$arguments)
 
   if (p2s) cat("-- Imputation", "1", "--")
@@ -348,16 +377,16 @@ disperse <- function(output, m = 5, dims = 1, p2s = 0, frontend=FALSE,...) {
   }
   flush.console()
 
-  # run EM, but return it with the theta at each iteration
+                                        # run EM, but return it with the theta at each iteration
   thetanew<-emarch(prepped$x,p2s=p2s,thetaold=NULL,tolerance=prepped$tolerance,startvals=0,priors=prepped$priors,empri=prepped$empri,frontend=frontend,allthetas=TRUE,collect=FALSE)  #change 4
 
-  # thetanew is a matrix whose columns are vectorized upper triangles of theta
-  # matrices for each iteration. thus, there are k(k+1)/2 rows.
+                                        # thetanew is a matrix whose columns are vectorized upper triangles of theta
+                                        # matrices for each iteration. thus, there are k(k+1)/2 rows.
   impdata<-thetanew$thetanew
 
-  # we'll put the theta of the last iteration into a new starting theta
+                                        # we'll put the theta of the last iteration into a new starting theta
   startsmat<-matrix(0,ncol(prepped$x)+1,ncol(prepped$x)+1)
-  startsmat[upper.tri(startsmat,TRUE)]<-c(-1,impdata[,ncol(impdata)])     
+  startsmat[upper.tri(startsmat,TRUE)]<-c(-1,impdata[,ncol(impdata)])
   startsmat<-t(startsmat)
   startsmat[upper.tri(startsmat,TRUE)]<-c(-1,impdata[,ncol(impdata)])
   iters<-nrow(thetanew$iter.hist)+1
@@ -367,10 +396,10 @@ disperse <- function(output, m = 5, dims = 1, p2s = 0, frontend=FALSE,...) {
     if (p2s) cat("-- Imputation", i, "--\n")
     if (frontend) {
       putAmelia("output.log", c(getAmelia("output.log"), paste("-- Imputation",i,"--\n")))
-    }    
+    }
 
-    # get a noisy sample of data from the that starting value (which is the
-    # Amelia answer) and use that to estimate a new starting theta (mus/vcov)
+                                        # get a noisy sample of data from the that starting value (which is the
+                                        # Amelia answer) and use that to estimate a new starting theta (mus/vcov)
     newstarts<-rmvnorm(round(2.5*ncol(prepped$x)),startsmat[1,2:ncol(startsmat)],startsmat[2:nrow(startsmat),2:nrow(startsmat)])
     startcov<-var(newstarts)
     startmus<-colMeans(newstarts)
@@ -379,41 +408,41 @@ disperse <- function(output, m = 5, dims = 1, p2s = 0, frontend=FALSE,...) {
     newstartsmat[1,2:nrow(startsmat)]<-startmus
     newstartsmat[2:nrow(startsmat),1]<-startmus
 
-    # grab the iteration history of the thetas
+                                        # grab the iteration history of the thetas
     thetanew<-emarch(prepped$x,p2s=p2s,thetaold=newstartsmat,tolerance=prepped$tolerance,startvals=0,priors=prepped$priors,empri=prepped$empri,frontend=frontend,allthetas=TRUE,collect=FALSE)  # change 5
     impdata<-cbind(impdata,thetanew$thetanew)
     iters<-c(iters,nrow(thetanew$iter.hist)+1)
   }
   if (dims==1)
     comps<-c(1)
-  else 
+  else
     comps<-c(1,2)
 
-  # reduce the dimenionality from k(k+1)/2 to 1 or 2 via principle components
+                                        # reduce the dimenionality from k(k+1)/2 to 1 or 2 via principle components
   rotations<-prcomp(t(impdata))$rotation[,comps]
   reduced.imps<-t(rotations)%*%impdata
 
   cols <- rainbow(m)
-  # plot the imputations
+                                        # plot the imputations
   if (frontend)
     x11()
   if (dims==1) {
-    addedroom<-(max(reduced.imps)-min(reduced.imps))*0.1    
+    addedroom<-(max(reduced.imps)-min(reduced.imps))*0.1
     x<-seq(iters[1])
     y<-reduced.imps[1,1:iters[1]]
     patt<-seq(1,length(x)-1)
     plot(x,y,col=1,main="Overdispersed Start Values",xlab="Number of Iterations",ylab="Largest Principle Component",xlim=c(0,max(iters)),ylim=range(c(reduced.imps-addedroom,reduced.imps)),type="n")
     segments(x[patt],y[patt],x[patt+1],y[patt+1],col=cols[1])
-    for (i in 2:length(iters)) {      
+    for (i in 2:length(iters)) {
       x<-seq(iters[i])
       y<-reduced.imps[1,(sum(iters[1:(i-1)])+1):(sum(iters[1:i]))]
-      patt<-seq(1,length(x)-1)                          
+      patt<-seq(1,length(x)-1)
       segments(x[patt],y[patt],x[patt+1],y[patt+1],col=cols[i])
-      #points(x,y,col=i)    
+                                        #points(x,y,col=i)
     }
     abline(h=reduced.imps[iters[1]],lwd=2)
     legend("bottomright",legend=c("Convergence of original starting values"),lwd=2,bty="n")
-   } else {
+  } else {
     xrange<-c((min(reduced.imps[1,])),(max(reduced.imps[1,])))
     yrange<-c((min(reduced.imps[2,])),(max(reduced.imps[2,])))
     plot(reduced.imps[1,1:iters[1]],reduced.imps[2,1:iters[1]],type="n",main="Overdispersed Starting Values",xlab="First Principle Component",ylab="Second Principle Component",col=cols[1],xlim=xrange,ylim=yrange)
@@ -430,7 +459,7 @@ disperse <- function(output, m = 5, dims = 1, p2s = 0, frontend=FALSE,...) {
       if (!is.null(patt))
         arrows(x[patt],y[patt],x[patt+1],y[patt+1],length=.1,col=cols[i])
       patt<-seq(1,length(x)-1)
-      segments(x[patt],y[patt],x[patt+1],y[patt+1],col=cols[i])    
+      segments(x[patt],y[patt],x[patt+1],y[patt+1],col=cols[i])
     }
     x<-reduced.imps[1,1:iters[1]]
     y<-reduced.imps[2,1:iters[1]]
@@ -442,76 +471,76 @@ disperse <- function(output, m = 5, dims = 1, p2s = 0, frontend=FALSE,...) {
     for (j in 1: length(xdiffs))
       if (veclength[j] > inchlength)
         patt<-c(patt,j)
-    #if (!is.null(patt))
-    #  arrows(x[patt],y[patt],x[patt+1],y[patt+1],length=.15,col=1,lwd=5)
-    patt<-seq(1,length(x)-1)                                   
+                                        #if (!is.null(patt))
+                                        #  arrows(x[patt],y[patt],x[patt+1],y[patt+1],length=.15,col=1,lwd=5)
+    patt<-seq(1,length(x)-1)
     segments(x[patt],y[patt],x[patt+1],y[patt+1],col=cols[1],lwd=1)
     dists<-gethull(st=impdata[,iters[1]],tol=prepped$tolerance,rots=rotations)
     convexhull<-chull(t(dists))
     convexhull<-c(convexhull,convexhull[1])
     lines(t(dists)[convexhull,],col="orange",pch=19,lwd=2)
     abline(h=0,lty=2)
-    abline(v=0,lty=2)  
+    abline(v=0,lty=2)
   }
-  #if (frontend)
-  #  tkdestroy(getAmelia("tcl.window"))
-  
+                                        #if (frontend)
+                                        #  tkdestroy(getAmelia("tcl.window"))
+
   invisible(list(impdata=impdata,p.order=prepped$p.order,index=prepped$index,iters=iters,rotations=rotations,dims=dims))
 
-}    
-   
+}
+
 
 sigalert<-function(data,disperse.list,output,notorious=5){
 
   k<-length(disperse.list$p.order)+1
 
-  # Construct Variable Names for all variables constructed in Imputation Model. 
-  # This uses the "index" which details all the variables included in the imputation model.
-  # The index is in the unstacked variable order.
-  # Possibly, if this is useful elsewhere, this might be moved to "prep.r".
+                                        # Construct Variable Names for all variables constructed in Imputation Model.
+                                        # This uses the "index" which details all the variables included in the imputation model.
+                                        # The index is in the unstacked variable order.
+                                        # Possibly, if this is useful elsewhere, this might be moved to "prep.r".
 
   varnm<-NULL
   lag.count<-0
   lead.count<-0
   poly.count<-0
   unknown.count<-0
-  
+
   for(i in 1:(k-1)){
     if(identical(disperse.list$index[i],-0.5)){
       lag.count<-lag.count+1
-      varnm<-c(varnm,paste("lag",lag.count))      
+      varnm<-c(varnm,paste("lag",lag.count))
     }else if(identical(disperse.list$index[i],0.5)){
       lead.count<-lead.count+1
-      varnm<-c(varnm,paste("lead",lead.count))      
+      varnm<-c(varnm,paste("lead",lead.count))
     }else if(identical(disperse.list$index[i],0)){
       poly.count<-poly.count+1
-      varnm<-c(varnm,paste("polytime",poly.count))      
+      varnm<-c(varnm,paste("polytime",poly.count))
     }else if(disperse.list$index[i]>=1){
       varnm<-c(varnm,names(data[disperse.list$index[i]]))      # Check what this does with matricies?
     }else{
       unknown.count<-unknown.count+1
-      varnm<-c(varnm,paste("unknown",unknown.count))      
+      varnm<-c(varnm,paste("unknown",unknown.count))
     }
   }
 
-#  WARNING: Currently assumes rotations is a vector.  If dim=2, rotations is a matrix.
-#  if(!identical(disperse.list$dims,1)){
-#    disperse.list$rotations<-disperse.list$rotations[1,]
-#  }
+                                        #  WARNING: Currently assumes rotations is a vector.  If dim=2, rotations is a matrix.
+                                        #  if(!identical(disperse.list$dims,1)){
+                                        #    disperse.list$rotations<-disperse.list$rotations[1,]
+                                        #  }
 
-  # This is a flag vector that identifies the largest values in the first principal component.
+                                        # This is a flag vector that identifies the largest values in the first principal component.
 
   largest.rotations<-disperse.list$rotations * 0
   largest.rotations[order(abs(disperse.list$rotations),decreasing=TRUE)[1:notorious]]<-1
-    
-  # This is a matrix of the size of theta, which has a 1 in the positions of the largest 
-  # contributions to the first principal component. 
-  # (largest corresponding elements of disperse.list$rotations)
+
+                                        # This is a matrix of the size of theta, which has a 1 in the positions of the largest
+                                        # contributions to the first principal component.
+                                        # (largest corresponding elements of disperse.list$rotations)
 
   map<-matrix(0,k,k)
-  map[upper.tri(map,TRUE)]<-c(0,largest.rotations)     
+  map[upper.tri(map,TRUE)]<-c(0,largest.rotations)
   map<-t(map)
-  map[upper.tri(map,TRUE)]<-c(0,largest.rotations) 
+  map[upper.tri(map,TRUE)]<-c(0,largest.rotations)
   map[c(1,disperse.list$p.order+1),c(1,disperse.list$p.order+1)]<-map                         # Rearrange to unstacked variable positions
 
   print(abs(map))
@@ -521,34 +550,34 @@ sigalert<-function(data,disperse.list,output,notorious=5){
   row.keep<-apply(map,1,gtz)
   col.keep<-apply(map,2,gtz)
 
-  # This is the submatrix of rotations, reshaped as a theta matrix, with the largest elements.
+                                        # This is the submatrix of rotations, reshaped as a theta matrix, with the largest elements.
 
   prcomp.matrix<-matrix(0,k,k)
-  prcomp.matrix[upper.tri(prcomp.matrix,TRUE)]<-c(0,disperse.list$rotations)     
+  prcomp.matrix[upper.tri(prcomp.matrix,TRUE)]<-c(0,disperse.list$rotations)
   prcomp.matrix<-t(prcomp.matrix)
-  prcomp.matrix[upper.tri(prcomp.matrix,TRUE)]<-c(0,disperse.list$rotations) 
+  prcomp.matrix[upper.tri(prcomp.matrix,TRUE)]<-c(0,disperse.list$rotations)
   prcomp.matrix[c(1,disperse.list$p.order+1),c(1,disperse.list$p.order+1)]<-prcomp.matrix     # Rearrange to unstacked variable positions
 
-  # This is the submatrix that we want to represent
+                                        # This is the submatrix that we want to represent
   portal<-prcomp.matrix[row.keep,col.keep]
   portalsize<-ncol(portal)
 
   portal.row.names<-varnm[row.keep]               # In symmetric matricies, these are the same.
   portal.col.names<-varnm[col.keep]               # In symmetric matricies, these are the same.
 
-  # This is a matrix that gives the relative rank of every element.
+                                        # This is a matrix that gives the relative rank of every element.
   col.map<-matrix(0,portalsize,portalsize)
   col.portal<-rank(abs(portal[upper.tri(portal,TRUE)]))
-  col.map[upper.tri(col.map,TRUE)]<-col.portal     
+  col.map[upper.tri(col.map,TRUE)]<-col.portal
   col.map<-t(col.map)
-  col.map[upper.tri(col.map,TRUE)]<-col.portal 
+  col.map[upper.tri(col.map,TRUE)]<-col.portal
 
-  # This creates a continuous color palette of the correct size.
+                                        # This creates a continuous color palette of the correct size.
   n.unique<-sum(upper.tri(matrix(1,portalsize,portalsize),TRUE))
   Lab.palette<-colorRampPalette(c("white","yellow","red"), space = "Lab")
   my.palette<-Lab.palette(n.unique)
 
-  # Plot the submatrix to be represented.
+                                        # Plot the submatrix to be represented.
 
   plot.new()
   plot.window(xlim=c(-2,portalsize+1), ylim=c(1,portalsize+3))
@@ -556,7 +585,7 @@ sigalert<-function(data,disperse.list,output,notorious=5){
     text(x=1,y=portalsize-i+1+0.5,pos=2,labels=portal.row.names[i])        # Row variable names
     for(j in 1:portalsize){
       rect(xleft=j, ybottom=portalsize-i+1, xright=j+1, ytop=portalsize-i+2, density = NULL, angle = 45,
-        col = my.palette[col.map[i,j]], border = NULL, lty = par("lty"), lwd = par("lwd"))
+           col = my.palette[col.map[i,j]], border = NULL, lty = par("lty"), lwd = par("lwd"))
       text(x=j+0.5,y=portalsize-i+1+0.5,labels=as.character(round(portal[i,j]*100)/100) )      # SHOULD FIND BETTER SIG FIGS HACK
     }
   }
@@ -569,8 +598,8 @@ sigalert<-function(data,disperse.list,output,notorious=5){
 
 
 tscsPlot <- function(output, var, cs, draws = 100, conf = .90,
-                      misscol = "red", obscol = "black", xlab, ylab, main,
-                      pch, ylim, xlim, frontend = FALSE, ...) {
+                     misscol = "red", obscol = "black", xlab, ylab, main,
+                     pch, ylim, xlim, frontend = FALSE, plotall=FALSE, nr, nc, pdfstub, ...) {
   if (missing(var))
     stop("I don't know which variable (var) to plot")
   if (missing(cs))
@@ -582,7 +611,7 @@ tscsPlot <- function(output, var, cs, draws = 100, conf = .90,
 
   data <- getOriginalData(output)
 
-  # Allow character names as arguments for "var" with data.frames
+                                        # Allow character names as arguments for "var" with data.frames
 
   if(is.character(var)){
     if(!is.data.frame(data)){
@@ -592,36 +621,52 @@ tscsPlot <- function(output, var, cs, draws = 100, conf = .90,
       if(is.na(varpos)){
         stop("the name provided for 'var' argument does not exist in the dataset provided")
       } else {
-      var<-varpos
+        var<-varpos
       }
     }
   }
 
-  units <- unique(data[,output$arguments$cs])
-  if (!(cs %in% units))
-    stop("the cross-section unit is not in the data")
+  units <- sort(unique(data[,output$arguments$cs]))
+  if(plotall){
+  	cs<-units
+  }else{
+    if (!(all(cs %in% units)))
+      stop("some cross-section unit requested for the plot is not in the data")
+  }
+
+# Picks a number of rows and columns if not user defined.  Maxs out at 4-by-4, unless user defined
+  	if(missing(nr)){
+  	  nr<-min(4,ceiling(sqrt(length(cs))))		
+  	}
+  	if(missing(nc)){
+  	  nc<-min(4,ceiling(length(cs)/nr))		
+	}
+
+  if(length(cs)>1){    	
+  	oldmfcol<-par()$mfcol
+    par(mfcol=c(nr,nc))
+  }
 
-  
   prepped <- amelia.prep(x = data, arglist=output$arguments)
   if (!is.null(prepped$blanks)) {
     data <- data[-prepped$blanks,]
-    unit.rows <- which(data[,output$arguments$cs]==cs)
+    unit.rows <- which(data[,output$arguments$cs] %in% cs)
     miss <- output$missMatrix[-prepped$blanks,][unit.rows, var] == 1
   } else {
-    unit.rows <- which(data[,output$arguments$cs]==cs)
+    unit.rows <- which(data[,output$arguments$cs] %in% cs)
     miss <- output$missMatrix[unit.rows, var] == 1
   }
- 
-  time <- data[unit.rows, output$arguments$ts]
 
-  cross.sec <- prepped$x[!is.na(match(prepped$n.order, unit.rows)),]
-  
+  time <- data[unit.rows, output$arguments$ts]   # These are the time values for rows appearing in some future plot
+  imps.cs<-data[unit.rows,output$arguments$cs]   # These are the cs units for rows appearing in some future plot
+  cross.sec <- prepped$x[!is.na(match(prepped$n.order, unit.rows)),]  
   stacked.var<-match(var,prepped$subset.index[prepped$p.order])
   subset.var<-match(var,prepped$subset.index)
   imps <- array(NA, dim=c(nrow(cross.sec), draws))
 
+
   drawsperimp <- (1/output$m)*draws
-  if (sum(miss) > 0) { 
+  if (sum(miss) > 0) {
     for (i in 1:draws) {
       currtheta <- output$theta[,,ceiling(i/drawsperimp)]
       imps[,i] <- amelia.impute(x = cross.sec, thetareal = currtheta,
@@ -629,7 +674,7 @@ tscsPlot <- function(output, var, cs, draws = 100, conf = .90,
                                 priors = prepped$priors,
                                 max.resample = output$arguments$max.resample)[,stacked.var]
     }
-    
+
     imps <- imps*prepped$scaled.sd[subset.var] + prepped$scaled.mu[subset.var]
     if (var %in% output$arguments$logs) {
       imps <- exp(imps)+prepped$xmin[which(var==output$arguments$logs)]
@@ -640,45 +685,125 @@ tscsPlot <- function(output, var, cs, draws = 100, conf = .90,
     if (var %in% output$arguments$lgstc) {
       imps <- exp(imps)/(1+exp(imps))
     }
-  
+
     outoforder <- match(prepped$n.order, unit.rows)[!is.na(match(prepped$n.order, unit.rows))]
     imps <- imps[order(outoforder),]
-    
-    means <- rowMeans(imps)
-
-    uppers <- apply(imps, 1, quantile, probs=(conf + (1 - conf)/2))
-    lowers <- apply(imps, 1, quantile, probs=(1-conf)/2)
-  } else {
-    means <- data[unit.rows, var]
-    uppers <- lowers <- means
   }
-  
 
   if (missing(pch)) pch <- 19
   if (missing(xlab)) xlab <- "time"
   if (missing(ylab)) ylab <- names(data)[var]
-  if (missing(main)) main <- cs
-  if (missing(xlim)) xlim <- range(time)
-  if (missing(ylim)) ylim <- range(c(uppers,lowers,means))
-  
-  cols <- ifelse(miss, misscol, obscol)
+
   if (frontend) {
     if (.Platform$OS.type == "windows") {
       windows()
     } else {
-      x11()      
+      x11()
     }
   }
-  plot(x = time, y = means, col = cols, pch = pch,ylim = ylim, xlim = xlim,
-       ylab = ylab, xlab = xlab, main = main, ...)
-  segments(x0 = time, x1 = time, y0 = lowers, y1 = uppers, col = cols, ...)
 
-  oiDetect <- (sum(output$missMatrix[unit.rows,var]) +
-               sum(!is.na(data[unit.rows, var]))) > length(unit.rows)
-  if (oiDetect) {
-    points(x = time, y = data[unit.rows, var], pch = pch, col = obscol)
+  count<-0
+  for(i in 1:length(cs)){
+
+      current.rows<- which(data[,output$arguments$cs]==cs[i])
+      current.time<- data[current.rows, output$arguments$ts]
+
+      flag<-imps.cs==cs[i]
+      current.miss<- miss[flag]
+
+      if(sum(current.miss)>0){                  
+      	  current.imps<-imps[flag,]
+      	  current.means <- rowMeans(current.imps)          
+      	  current.uppers <- apply(current.imps, 1, quantile, probs=(conf + (1 - conf)/2))   # THIS IS LIKELY SLOW
+      	  current.lowers <- apply(current.imps, 1, quantile, probs=(1-conf)/2)              # THIS IS LIKELY SLOW
+  	  } else {
+      	  current.means <- data[current.rows, var]
+    	  current.uppers <- current.lowers <- current.means
+  	  }
+
+      cols <- ifelse(current.miss, misscol, obscol)
+      current.main<-ifelse(missing(main), cs[i], main)  # Allow title to be rolling if not defined
+      if(missing(xlim)){                                # Allow axes to vary by unit, if not defined
+      	current.xlim<-range(current.time)
+      }else{
+      	current.xlim<-xlim
+      }
+      if(missing(ylim)){
+      	current.ylim<-range(current.uppers,current.lowers,current.means)
+      }else{
+      	current.ylim<-ylim
+      }
+      
+      plot(x = current.time, y = current.means, col = cols, pch = pch, ylim = current.ylim, xlim = current.xlim,
+         ylab = ylab, xlab = xlab, main = current.main, ...)
+      segments(x0 = current.time, x1 = current.time, y0 = current.lowers, y1 = current.uppers, col = cols, ...)
+
+      oiDetect <- (sum(output$missMatrix[current.rows,var]) +
+               sum(!is.na(data[current.rows, var]))) > length(current.rows)
+      if (oiDetect) {
+        points(x = current.time, y = data[current.rows, var], pch = pch, col = obscol)
+      }
+
+      # print page if window full
+	  if((!missing(pdfstub)) & (i %% (nr*nc) ==0)){
+	  	count<-count+1
+	  	dev.copy2pdf(file=paste(pdfstub,count,".pdf",sep=""))
+	  }
   }
-    
+
+  if(!missing(pdfstub)){ 
+    if(i %% (nr*nc) !=0){           # print last page if not complete
+	    count<-count+1
+	    dev.copy2pdf(file=paste(pdfstub,count,".pdf",sep=""))
+    }
+    par(mfcol=oldmfcol)             # return to previous windowing 
+  }                                 # although always now fills by col even if previously by row
+ 
   invisible(imps)
-  
+
+}
+
+##
+## mi.meld
+##
+## combines estimated quantities of interest and their standard errors
+##   from multiply imputed datasets
+## Default expects the q's (and their se's) to have all parameters be columns
+##   in a matrix, and the results from each imputed dataset to be a new
+##   row.
+##
+##  INPUTS: q  - matrix of quantities of interest (default m by k, thus each row
+##          represents k parameters from one of m different datasets)
+##          se - matrix of standard errors (default m by k)
+##          byrow - logical.  TRUE: matrix is m-by-k.  FALSE: matrix is k-by-m.
+##  2/21/12 jH
+##
+
+
+mi.meld<-function(q, se, byrow = TRUE) {
+
+  if (!byrow) {
+    q <- t(q)
+    se <- t(se)
+  }
+
+  if (is.data.frame(q)) {
+    q <- as.matrix(q)
+  }
+
+  if (is.data.frame(se)) {
+    se <- as.matrix(se)
+  }
+
+  am.m <- nrow(q)
+
+  ones <- matrix(1, nrow = 1, ncol = am.m)
+  imp.q <- (ones %*% q)/am.m        # Slightly faster than "apply(b,2,mean)"
+  ave.se2 <- (ones %*% (se^2))/am.m # Similarly, faster than "apply(se^2,2,mean)"
+  diff <- q - matrix(1, nrow = am.m, ncol = 1) %*% imp.q
+  sq2 <- (ones %*% (diff^2))/(am.m - 1)
+  imp.se <- sqrt(ave.se2 + sq2 * (1 + 1/am.m))
+
+  return(list(q.mi = imp.q, se.mi = imp.se))
+
 }
